<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Defines the editing form for the quim question type.
 * Based on short answer.
 * @package    qtype
 * @subpackage quim
 * @copyright  2007 Jamie Pratt
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


defined('MOODLE_INTERNAL') || die();


/**
 * Quim question editing form definition.
 *
 * @copyright  2007 Jamie Pratt
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_quim_edit_form extends question_edit_form {

    protected function definition_inner($mform) {
        $menu = array(
            get_string('caseno', 'qtype_quim'),
            get_string('caseyes', 'qtype_quim')
        );
        $mform->addElement('select', 'usecase',
                get_string('casesensitive', 'qtype_quim'), $menu);

        $mform->addElement('static', 'answersinstruct',
                get_string('correctanswers', 'qtype_quim'),
                get_string('filloutoneanswer', 'qtype_quim'));
        $mform->closeHeaderBefore('answersinstruct');

        $this->add_per_answer_fields($mform, get_string('answerno', 'qtype_quim', '{no}'),
                question_bank::fraction_options());

        $this->add_interactive_settings();
    }

    protected function data_preprocessing($question) {
        $question = parent::data_preprocessing($question);
        //$question = $this->data_preprocessing_answers($question);
		$question = $this->data_preprocessing_answers($question, true);
        $question = $this->data_preprocessing_hints($question);

        return $question;
    }

	//New validation function.
	public function validation($data, $files) {
		$errors = parent::validation($data, $files);
        $answers = $data['answer'];
        $answercount = 0;
        $maxgrade = false;
		
		//answer es un array de 2 elements, nomes interessa el primer.
		foreach($answers as $key => $answer) {
			if(is_array($answer)) {
				$answer = $answer['text'];
			}
			
			$answer = preg_replace('/\s+/', '', $answer); //eliminar espais
			$answer = preg_replace("/<(\/)?p>/u", '', $answer); //elimina <p> i </p> tags
			//echo '<span>';
			//echo $answer;
			//echo '</span>';
			$trimmedanswer = trim($answer);
			//echo '<span>';
			//echo $trimmedanswer;
			//echo '</span>';
			if ($trimmedanswer !== '') {
                $answercount++;
                if ($data['fraction'][$key] == 1) {
                    $maxgrade = true;
                }
            } else if ($data['fraction'][$key] != 0 ||
                    !html_is_blank($data['feedback'][$key]['text'])) {
                $errors["answer[$key]"] = get_string('answermustbegiven', 'qtype_quim');
                $answercount++;
            }
        }
        if ($answercount==0) {
            $errors['answer[0]'] = get_string('notenoughanswers', 'qtype_quim', 1);
        }
        if ($maxgrade == false) {
            $errors['fraction[0]'] = get_string('fractionsnomax', 'question');
        }
        return $errors;
    }
	
	
    public function validation2($data, $files) {
        $errors = parent::validation($data, $files);
        $answers = $data['answer'];
        $answercount = 0;
        $maxgrade = false;
		
		foreach ($answers as $key => $answer) {

			/*echo gettype($answer);
			$fitxer = "answer.txt";
			$fh = fopen($fitxer, 'w+');
			fwrite($fh, $answer);
			fclose($fh);*/

            $trimmedanswer = trim($answer);
			/*echo gettype($trimmedanswer);
			$fitxer = "answertrimmed.txt";
			$fh = fopen($fitxer, 'w+');
			fwrite($fh, $trimmedanswer);
			fclose($fh);*/
			
            if ($trimmedanswer !== '') {
                $answercount++;
                if ($data['fraction'][$key] == 1) {
                    $maxgrade = true;
                }
            } else if ($data['fraction'][$key] != 0 ||
                    !html_is_blank($data['feedback'][$key]['text'])) {
                $errors["answer[$key]"] = get_string('answermustbegiven', 'qtype_quim');
                $answercount++;
            }
        }
        if ($answercount==0) {
            $errors['answer[0]'] = get_string('notenoughanswers', 'qtype_quim', 1);
        }
        if ($maxgrade == false) {
            $errors['fraction[0]'] = get_string('fractionsnomax', 'question');
        }
        return $errors;
    }

	
	/**
     * Get the list of form elements to repeat, one for each answer.
     * @param object $mform the form being built.
     * @param $label the label to use for each option.
     * @param $gradeoptions the possible grades for each answer.
     * @param $repeatedoptions reference to array of repeated options to fill
     * @param $answersoption reference to return the name of $question->options
     *      field holding an array of answers
     * @return array of form fields.
     */
    protected function get_per_answer_fields($mform, $label, $gradeoptions,
            &$repeatedoptions, &$answersoption) {
			
			
			//$parentansweroptions = parent::get_per_answer_fields($mform, $label, $gradeoptions,
            //                                            $repeatedoptions, $answersoption);
			//print_r($parentansweroptions);
			//print_r($label);
        $repeated = array();
        $repeated[] = $mform->createElement('header', 'answerhdr', $label);
		//$repeated[] = $parentansweroptions[1];
        $repeated[] = $mform->createElement('editor', 'answer',
                get_string('answer', 'question'), array('size' => 80));
        $repeated[] = $mform->createElement('select', 'fraction',
                get_string('grade'), $gradeoptions);
        $repeated[] = $mform->createElement('editor', 'feedback',
                get_string('feedback', 'question'), array('rows' => 5), $this->editoroptions);
        $repeatedoptions['answer']['type'] = PARAM_RAW;
        $repeatedoptions['fraction']['default'] = 0;
        $answersoption = 'answers';
        return $repeated;
    }
	
	public function set_data($question) {
		//print_r($question);
		//print_r($question->options->answers);
/*		foreach($question->options->answers as $ans) {
			echo $ans->answer;
		}
*/
		parent::set_data($question);
	}
	
    public function qtype() {
        return 'quim';
    }
}
