﻿<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * quim question definition class based on short answer.
 *
 * @package    qtype
 * @subpackage quim
 * @copyright  2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

mb_internal_encoding('UTF-8');
mb_regex_encoding('UTF-8');
defined('MOODLE_INTERNAL') || die();


/**
 * Represents a quim question.
 *
 * @copyright  2009 The Open University
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class qtype_quim_question extends question_graded_by_strategy
        implements question_response_answer_comparer {
    /** @var boolean whether answers should be graded case-sensitively. */
    public $usecase;
    /** @var array of question_answer. */
    public $answers = array();

    public function __construct() {
        parent::__construct(new question_first_matching_answer_grading_strategy($this));
    }

    public function get_expected_data() {
        return array('answer' => PARAM_RAW_TRIMMED);
    }

    public function summarise_response(array $response) {
        if (isset($response['answer'])) {
            return $response['answer'];
        } else {
            return null;
        }
    }

    public function is_complete_response(array $response) {
        return array_key_exists('answer', $response) &&
                ($response['answer'] || $response['answer'] === '0');
    }

    public function get_validation_error(array $response) {
        if ($this->is_gradable_response($response)) {
            return '';
        }
        return get_string('pleaseenterananswer', 'qtype_quim');
    }

    public function is_same_response(array $prevresponse, array $newresponse) {
        return question_utils::arrays_same_at_key_missing_is_blank(
                $prevresponse, $newresponse, 'answer');
    }

    public function get_answers() {
        return $this->answers;
    }

    public function compare_response_with_answer(array $response, question_answer $answer) {

		return self::compare_response_with_correct_answer($response['answer'],$answer->answer);
		
		
		/*
		if (!array_key_exists('answer', $response) || is_null($response['answer'])) {
            return false;
        }

        return self::compare_string_with_wildcard(
                $response['answer'], $answer->answer, !$this->usecase);
		*/
    }

	/**
	 * Returns true if response matches the correct_answer.
	 */
	public static function compare_response_with_correct_answer($response, $correct_answer) {
		
		//Split answer to obtain single expression elements
		$ans1 = self::prepare_answer($response);
		$ans2 = self::prepare_answer($correct_answer);
		print_r($ans1);
		echo'<br>';
		print_r($ans2);
		echo'<br>';
		//Response has errors. (empty, more than one equal)
		if($ans1 == null) { return false; }
	
		//Different number of members between response and and correct answer
		if($ans1[0] != $ans2[0]) { return false; }
		
		//If previous tests are OK both expressions are parsed and compared
		if(self::compare($ans1,$ans2)) { return true; }

	}	
	
	public static function is_subsup($exp,$i) {
	if($i+4 < count($exp)) {
		$subsup = $exp[$i].$exp[$i+1].$exp[$i+2].$exp[$i+3].$exp[$i+4];
		if($subsup == '<sub>' or $subsup == '<sup>') return true;
	}
	return false;
	} 

	public static function closes_subsup($exp,$i) {
		if($i+5 < count($exp)) {
			$subsup = $exp[$i].$exp[$i+1].$exp[$i+2].$exp[$i+3].$exp[$i+4].$exp[$i+5];
			if($subsup == '</sub>' or $subsup == '</sup>') return true;
		}
		return false;
	} 

	public static function parse($expressio) {

		$exp = preg_split('//u', $expressio, -1, PREG_SPLIT_NO_EMPTY);

		//per cada caracter, es pot separar del conjunt?
		$lst = array();
		$ops = '';
		if($exp[0] == '+' or $exp[0] == '-') $ops .= $exp[0];
		else $ops .= '+'.$exp[0];
		$i = 1;
		$separable = true;
		while($i < count($exp)) {
			if($exp[$i] == '(' and $separable) { //find close
				$ops .= '(';
				++$i;
				$separable = false;
				$opened = 1;
				while($i < count($exp) and $opened != 0) {
					if($exp[$i] == ')') --$opened;
					else if($exp[$i] == '(') ++$opened;
					$ops .= $exp[$i];
					++$i;
				}	
				
				$separable = true;
			}
			else if(($exp[$i] == '+' or $exp[$i] == '-') and $separable) { //separable if not in parentheses or <sub><sup>
				$lst[] = $ops;
				$ops = $exp[$i];
				++$i;
			}
			
			else if( ($exp[$i] == '<') and self::is_subsup($exp,$i) ) {
				for($j=$i; $j<$i+5;++$j) {
					$ops .= $exp[$j];
				}
				$i += 5;
				$closed = false;
				while($i < count($exp) and !$closed) {
					if($exp[$i] != '<') {
						$ops .= $exp[$i];
						++$i;
					}
					else if(self::closes_subsup($exp,$i)) {
						$closed = true;
						for($j=$i; $j<$i+6;++$j) {
							$ops .= $exp[$j];
						}
						$i += 6;
					}
					else ++$i;
				}
			}
			else {
				$ops .= $exp[$i];
				++$i;
			}
		}
		
		if(mb_strlen($ops) != 0) $lst[] = $ops;
		
		return $lst;
		
	}

	public static function compare($ans1, $ans2) {
		if($ans1[0] == 1) {//una sola expressio
			$ans1 = $ans1[1];
			$ans2 = $ans2[1];
			return self::equals($ans1,$ans2);
		}
		
		else if($ans1[0] == 2) { //equacio
			$ans1 = $ans1[1];
			$ans2 = $ans2[1];
			foreach($ans1 as $costat1) {
				$found = false;
				foreach($ans2 as $key=>$costat2) {
					if(self::equals($costat1,$costat2)) {
						$ans2[$key] = null;
						$found = true;
						break;
					}
				}
				if(!$found) return false;
			}
			
			return true;
					
		}
		
		return false;

	}

	public static function equals($ans1,$ans2) {
		$size1 = count($ans1);
		$size2 = count($ans2);
		if($size1 != $size2) return false;

		for($i = 0; $i < $size1; ++$i) {
			$found = false;
			for($j = 0; $j < $size1; ++$j) {
				if($ans1[$i] == $ans2[$j]) {
					$ans2[$j] = null;
					$found = true;
					break;
				}
			}
			if(!$found) return false;
		}
		return true;
	}


	//returns array. array[0] contains 1 or 2 if single expression or equation
	public static function prepare_answer($expr) {
		$ops = array();

		//trim all spaces
		$expr = preg_replace('/\s+/u', '', $expr);
		$expr = preg_replace("/<(\/)?p>/u", '', $expr);
		//splits if has allowed equals
		$ops = preg_split( "/(=|≅|≈|≠|≡|←|→|↔|⇐|⇒|⇔)/u", $expr);
		//empty. makes no sense.
		if(count($ops) == 0) {
			return null;
		}
		//single expression no equals no equation
		else if(count($ops) == 1) {
			$lst = Array();
			$lst[] = 1;
			$lst[] = self::parse($ops[0]);
			return $lst;
		}
		//too many equals. not allowed by now
		else if(count($ops) > 2) {
			return null;
		}
		//two equals
		else {
			
			$operador = mb_substr($expr, mb_strlen($ops[0]), 1,"utf-8");
			
			$answer  = array();
			$answer[] = 2;
			$temp = array();
			foreach($ops as $op) {

				$lst = self::parse($op);
				
				$temp[] = $lst;
			}
			$answer[] = $temp;
			return $answer;
			
		}

	}
	
	
	
	
	
	
	
	
	
	
	
	
    public static function compare_string_with_wildcard($string, $pattern, $ignorecase) {
        // Break the string on non-escaped asterisks.
        $bits = preg_split('/(?<!\\\\)\*/', $pattern);
        // Escape regexp special characters in the bits.
        $excapedbits = array();
        foreach ($bits as $bit) {
            $excapedbits[] = preg_quote(str_replace('\*', '*', $bit));
        }
        // Put it back together to make the regexp.
        $regexp = '|^' . implode('.*', $excapedbits) . '$|u';

        // Make the match insensitive if requested to.
        if ($ignorecase) {
            $regexp .= 'i';
        }

        if (function_exists('normalizer_normalize')) {
            $regexp = normalizer_normalize($regexp, Normalizer::FORM_C);
            $string = normalizer_normalize($string, Normalizer::FORM_C);
        }

        return preg_match($regexp, trim($string));
    }

    public function get_correct_response() {
        $response = parent::get_correct_response();
        if ($response) {
            $response['answer'] = $this->clean_response($response['answer']);
        }
        return $response;
    }

    public function clean_response($answer) {
        // Break the string on non-escaped asterisks.
        $bits = preg_split('/(?<!\\\\)\*/', $answer);

        // Unescape *s in the bits.
        $cleanbits = array();
        foreach ($bits as $bit) {
            $cleanbits[] = str_replace('\*', '*', $bit);
        }

        // Put it back together with spaces to look nice.
        return trim(implode(' ', $cleanbits));
    }

    public function check_file_access($qa, $options, $component, $filearea,
            $args, $forcedownload) {
        if ($component == 'question' && $filearea == 'answerfeedback') {
            $currentanswer = $qa->get_last_qt_var('answer');
            $answer = $this->get_matching_answer(array('answer' => $currentanswer));
            $answerid = reset($args); // itemid is answer id.
            return $options->feedback && $answer && $answerid == $answer->id;

        } else if ($component == 'question' && $filearea == 'hint') {
            return $this->check_hint_file_access($qa, $options, $args);

        } else {
            return parent::check_file_access($qa, $options, $component, $filearea,
                    $args, $forcedownload);
        }
    }
}
